#-----------------------------------------------------------------------------
#
# Thomas Thomassen
# thomas[at]thomthom[dot]net
#
#-----------------------------------------------------------------------------

require 'sketchup.rb'

#-----------------------------------------------------------------------------

module TT_Raytrace

  def self.raytest(ray)
    model = Sketchup.active_model
    vector = ray[1]
    while true
      # Shot ray
      test = model.raytest(ray)
      return nil if test.nil?
      # Verify visibility
      point, path = test
      if path.all? { |e| e.visible? && e.layer.visible? }
        return test
      else
        # raytest stopped on a hidden entity - create a new ray from that point
        # and let the loop continue until it hits something visible, or nothing.
        ray = [ point, vector ]
      end
    end
  end
  
  def self.drop_cpoints(native = false)
    model = Sketchup.active_model
    sel = model.selection
    
    model.start_operation('Drop CPoints')
    
    group = model.active_entities.add_group
    group.material = (native) ? 'red' : 'green'
    sel.each { |e|
      next unless e.is_a?(Sketchup::ConstructionPoint)
      ray = [e.position, Z_AXIS.reverse]
      if native
        result = model.raytest(ray)
      else
        result = self.raytest(ray)
      end
      unless result.nil?
        #model.active_entities.add_cpoint(result[0])
        #model.active_entities.add_cline(e.position, result[0])
        group.entities.add_cpoint(result[0])
        group.entities.add_line(e.position, result[0])
      end
    }
    
    model.commit_operation
  end

end # module